@@ -1,4 +1,4 @@
-/* $Id: bogoconfig.c,v 1.234 2005/11/05 04:04:40 relson Exp $ */
+/* $Id: bogoconfig.c,v 1.245 2006/11/23 15:44:17 relson Exp $ */
 
 /*****************************************************************************
 
@@ -36,8 +36,6 @@ Note: bogolexer also uses configfile.c.
 #include "common.h"
 
 #include <ctype.h>
-#include <unistd.h>
-#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
@@ -94,7 +92,9 @@ static void comma_parse(char opt, const char *arg, double *parm1, double *parm2,
 static struct option longopts_bogofilter[] = {
     /* longoptions.h - common options */
     LONGOPTIONS_COMMON
-    /* longoptions.h - bogofilter/-lexer options */
+    /* longoptions.h - bogofilter */
+    LONGOPTIONS_MAIN
+    /* longoptions.h - bogofilter/bogolexer options */
     LONGOPTIONS_LEX
     /* longoptions.h - bogofilter/bogoutil options */
     LONGOPTIONS_DB
@@ -325,13 +325,24 @@ static const char *help_text[] = {
     "  --bogofilter-dir                  directory for wordlists\n",
     "  --charset-default                 default character set\n",
     "  --db-cachesize                    Berkeley db cache in Mb\n",
+#ifdef	HAVE_DECL_DB_CREATE
+    "  --db-log-autoremove               enable/disable autoremoval of log files\n",
+    "  --db-transaction                  enable/disable transactions\n",
+ #ifdef	FUTURE_DB_OPTIONS
+    "  --db-txn-durable                                 \n",
+ #endif
+#endif
     "  --ham-cutoff                      nonspam if score below this\n",
     "  --header-format                   spam header format\n",
     "  --log-header-format               header written to log\n",
     "  --log-update-format               logged on update\n",
     "  --min-dev                         ignore if score near\n",
+    "  --min-token-len                   min len for single tokens\n",
+    "  --max-token-len                   max len for single tokens\n",
+    "  --max-multi-token-len             max len for multi-word tokens\n",
+    "  --multi-token-count               number of tokens per multi-word token\n",
     "  --ns-esf                          effective size factor for ham\n",
-    "  --replace-nonascii-characters     substitute '?' if bit8 is 1\n",
+    "  --replace-nonascii-characters     substitute '?' if bit 8 is 1\n",
     "  --robs                            Robinson's s parameter\n",
     "  --robx                            Robinson's x parameter\n",
     "  --sp-esf                          effective size factor for spam\n",
@@ -344,9 +355,9 @@ static const char *help_text[] = {
     "  --terse                           report in short form\n",
     "  --terse-format                    short form\n",
     "  --thresh-update                   no update if near 0 or 1\n",
-    "  --timestamp                       apply token timestamps\n",
+    "  --timestamp                       enable/disable token timestamps\n",
 #ifndef	DISABLE_UNICODE
-    "  --unicode=yes/no                  enable/disable unicode based wordlist\n",
+    "  --unicode                         enable/disable unicode based wordlist\n",
 #endif
     "  --unsure-subject-tag              like spam-subject-tag\n",
     "  --user-config-file                configuration file\n",
@@ -378,8 +389,8 @@ static void print_version(void)
     (void)fprintf(stdout,
 		  "%s version %s\n"
 		  "    Database: %s\n"
-		  "Copyright (C) 2002-2005 Eric S. Raymond,\n"
-		  "    David Relson, Matthias Andree, Greg Louis\n\n"
+		  "Copyright (C) 2002-2006 David Relson, Matthias Andree, Greg Louis\n"
+		  "Copyright (C) 2002-2003 Eric S. Raymond, Adrian Otto, Gyepi Sam.\n\n"
 		  "%s comes with ABSOLUTELY NO WARRANTY.  "
 		  "This is free software, and\nyou are welcome to "
 		  "redistribute it under the General Public License.  "
@@ -389,7 +400,7 @@ static void print_version(void)
 		  progtype, version, ds_version_str(), PACKAGE);
 }
 
-#define	OPTIONS	":-:bBc:Cd:DehHI:k:lL:m:MnNo:O:pqQRsStTuUvVx:X:y:"
+#define	OPTIONS	":-:bBc:Cd:DehHI:k:lL:m:MnNo:O:pPqQRsStTuUvVx:X:y:"
 
 /** These functions process command line arguments.
  **
@@ -509,6 +520,7 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	break;
 
     case 'e':
+    case O_HAM_TRUE:
 	nonspam_exits_zero = true;
 	break;
 
@@ -696,6 +708,10 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
     case O_HEADER_FORMAT:		header_format = get_string(name, val);			break;
     case O_LOG_HEADER_FORMAT:		log_header_format = get_string(name, val);		break;
     case O_LOG_UPDATE_FORMAT:		log_update_format = get_string(name, val);		break;
+    case O_MAX_TOKEN_LEN:		max_token_len=atoi(val);				break;
+    case O_MIN_TOKEN_LEN:		min_token_len=atoi(val);				break;
+    case O_MAX_MULTI_TOKEN_LEN:		max_multi_token_len=atoi(val);				break;
+    case O_MULTI_TOKEN_COUNT:		multi_token_count=atoi(val);				break;
     case O_REPLACE_NONASCII_CHARACTERS:	replace_nonascii_characters = get_bool(name, val);	break;
     case O_SPAMICITY_FORMATS:		set_spamicity_formats(val);				break;
     case O_SPAMICITY_TAGS:		set_spamicity_tags(val);				break;
