@@ -1,4 +1,4 @@
-/* $Id: bogohist.h,v 1.1 2004/01/03 14:09:03 relson Exp $ */
+/* $Id: bogohist.h,v 1.3 2005/04/04 11:16:19 relson Exp $ */
 
 /*****************************************************************************
 
@@ -13,7 +13,9 @@ AUTHOR:
 #ifndef	HISTOGRAM_H
 #define	HISTOGRAM_H
 
-extern int histogram(const char *path);
+#include "paths.h"
+
+extern ex_t histogram(bfpath *bfp);
 
 #endif	/* HISTOGRAM_H */
 
