@@ -1,4 +1,4 @@
-/* $Id: bogolexer.c,v 1.94 2006/04/29 21:11:58 relson Exp $ */
+/* $Id: bogolexer.c,v 1.98 2006/07/02 21:06:48 relson Exp $ */
 
 /*****************************************************************************
 
@@ -256,6 +256,22 @@ void process_arg(int option, const char *name, const char *val, priority_t prece
 	block_on_subnets = get_bool(name, val);
 	break;
 
+    case O_MAX_TOKEN_LEN:
+	max_token_len = atoi(val);
+	break;
+
+    case O_MIN_TOKEN_LEN:
+	min_token_len = atoi(val);
+	break;
+
+    case O_MAX_MULTI_TOKEN_LEN:
+	max_multi_token_len=atoi(val);
+	break;
+
+    case O_MULTI_TOKEN_COUNT:
+	multi_token_count=atoi(val);
+	break;
+
     default:
 	/* config file options:
 	**  ok    - if from config file
@@ -297,23 +313,26 @@ int main(int argc, char **argv)
     bogoreader_init(argc, argv);
 
     while ((*reader_more)()) {
-	word_t *token;
+	word_t token;
 	lexer_init();
 
 	while ((t = get_token( &token )) != NONE)
 	{
 	    count += 1;
 	    if (passthrough) {
-		fprintf(fpo, "%s\n", token->text);
+		fprintf(fpo, "%s\n", token.text);
 	    }
 	    else if (!quiet)
-		fprintf(fpo, "get_token: %d \"%s\"\n", (int)t, token->text);
+		fprintf(fpo, "get_token: %d \"%s\"\n", (int)t, token.text);
 	}
     }
 
     if ( !passthrough )
 	fprintf(fpo, "%d tokens read.\n", count);
 
+    /* cleanup storage */
+    token_cleanup();
+    mime_cleanup();
     textblock_free();
 
     MEMDISPLAY;
