@@ -1,4 +1,4 @@
-/* $Id: bogoutil.c,v 1.233 2006/05/29 14:28:00 relson Exp $ */
+/* $Id: bogoutil.c,v 1.240 2006/11/23 15:44:17 relson Exp $ */
 
 /*****************************************************************************
 
@@ -202,7 +202,8 @@ static ex_t load_wordlist(bfpath *bfp)
 	p = spanword(buf);
 	len = strlen((const char *)buf);
 
-	if (len > MAXTOKENLEN)
+	if (max_token_len != 0 &&
+	    len > max_token_len)
 	    continue;		/* too long - discard */
 
 	spamcount = (uint) atoi((const char *)p);
@@ -462,7 +463,8 @@ static void print_version(void)
     (void)fprintf(stdout,
 		  "%s version %s\n"
 		  "    Database: %s\n"
-		  "Copyright (C) 2002-2006 Gyepi Sam, David Relson, Matthias Andree\n\n"
+		  "Copyright (C) 2002-2006 David Relson, Matthias Andree\n"
+		  "Copyright (C) 2002-2003 Gyepi Sam.\n\n"
 		  "%s comes with ABSOLUTELY NO WARRANTY.  "
 		  "This is free software, and\nyou are welcome to "
 		  "redistribute it under the General Public License.  "
@@ -540,6 +542,14 @@ static const char *help_text[] = {
     "  --unicode=yes/no            - convert wordlist to/from unicode\n",
 #endif
     "\n",
+
+    "token parsing options:\n",
+    "  --min-token-len             - min len for single tokens\n",
+    "  --max-token-len             - max len for single tokens\n",
+    "  --max-multi-token-len       - max len for multi-word tokens\n",
+    "  --multi-token-count         - number of tokens per multi-word token\n",
+    "\n",
+
     NULL
     };
 
@@ -604,6 +614,9 @@ static int process_arglist(int argc, char **argv)
     _wildcard (&argc, &argv);	/* expand wildcards (*.*) */
 #endif
 
+    /* default: no token length checking */
+    max_token_len = 0;
+
     while (1)
     {
 	int option_index = 0;
@@ -620,6 +633,14 @@ static int process_arglist(int argc, char **argv)
 	count += process_arg(option, name, optarg);
     }
 
+    if (max_token_len != 0 && max_multi_token_len == 0) {
+	/* token length checking ... */
+	if (multi_token_count == 1)
+	    max_multi_token_len = max_token_len + MAX_PREFIX_LEN;
+	else
+	    max_multi_token_len = max_token_len = (max_token_len+1) * multi_token_count + MAX_PREFIX_LEN;
+    }
+
     if (count != 1)
     {
 	usage(stderr);
@@ -811,6 +832,22 @@ static int process_arg(int option, const char *name, const char *val)
 	encoding = str_to_bool(val) ? E_UNICODE : E_RAW;
 	break;
 
+    case O_MAX_TOKEN_LEN:
+	max_token_len = atoi(optarg);
+	break;
+
+    case O_MIN_TOKEN_LEN:
+	min_token_len = atoi(optarg);
+	break;
+
+    case O_MAX_MULTI_TOKEN_LEN:
+	max_multi_token_len=atoi(optarg);
+	break;
+
+    case O_MULTI_TOKEN_COUNT:
+	multi_token_count=atoi(optarg);
+	break;
+
     default:
 	if (!dsm_options_bogoutil(option, &flag, &count, &ds_file, name, val)) {
 	    fprintf(stderr, "Invalid option '%s'\n", name);
