@@ -1,4 +1,4 @@
-/* $Id: configfile.c,v 1.47 2004/08/22 15:52:28 relson Exp $ */
+/* $Id: configfile.c,v 1.53 2005/11/18 23:15:43 m-a Exp $ */
 
 /*****************************************************************************
 
@@ -29,6 +29,7 @@ AUTHOR:
 #include "find_home.h"
 #include "format.h"
 #include "lexer.h"
+#include "longoptions.h"
 #include "maint.h"
 #include "wordlists.h"
 #include "xatox.h"
@@ -73,54 +74,76 @@ void remove_comment(const char *line)
     return;
 }
 
-bool process_config_option(const char *arg, bool warn_on_error, priority_t precedence)
+bool process_config_option(const char *arg, bool warn_on_error, priority_t precedence, struct option *longopts)
 {
     uint pos;
     bool ok = true;
 
     char *val = NULL;
-    char *opt = xstrdup(arg);
+    const char *opt = arg;
+    char *dupl;
     const char delim[] = " \t=";
 
-    pos = strcspn(arg, delim);
-    if (pos < strlen(arg)) { 		/* if delimiter present */
-	val = opt + pos;
+    while (isspace(*opt))		/* ignore leadign whitespace */
+	opt += 1;
+
+    dupl = xstrdup(opt);
+    pos = strcspn(dupl, delim);
+    if (pos < strlen(dupl)) { 		/* if delimiter present */
+	val = dupl + pos;
 	*val++ = '\0';
 	val += strspn(val, delim);
     }
 
-    if (val == NULL || 
-	!process_config_option_as_arg(opt, val, precedence)) {
+    if (val == NULL ||
+	!process_config_option_as_arg(dupl, val, precedence, longopts)) {
 	ok = false;
 	if (warn_on_error)
 	    fprintf(stderr, "Error - bad parameter '%s'\n", arg);
     }
 
-    xfree(opt);
+    xfree(dupl);
     return ok;
 }
 
-bool process_config_option_as_arg(const char *opt, const char *val, priority_t precedence)
-{
-    int idx = 0;
-    size_t len = strlen(opt);
+/* option_compare()
+**
+** Returns true if options are equal, without regard to case and
+** allows underscore from config file to match hyphen
+*/
 
-    for (idx = 0; ; idx += 1) {
-	const char *s = long_options[idx].name;
-	if (s == NULL)
-	    break;
-	if (strlen(s) != len || strncasecmp(s, opt, len) != 0)
+static bool option_compare(const char *opt, const char *name)
+{
+    char co, cn;
+    if (strlen(opt) != strlen(name))
+	return false;
+    while (((co = *opt++) != '\0') && ((cn = *name++) != '\0')) {
+	if ((co == cn) || (tolower(co) == tolower(cn)))
+	    continue;
+	if (co != '_' || cn != '-')
+	    return false;
+    }
+    return true;
+}
+
+bool process_config_option_as_arg(const char *opt, const char *val, priority_t precedence, struct option *long_options)
+{
+    struct option *option;
+
+    for (option = long_options; option->name; option += 1) {
+	if (!option_compare(opt, option->name))
 	    continue;
 	if (strcmp(val, "''") == 0)
 	    val = "";
-	process_arg(long_options[idx].val, s, val, precedence, PASS_2_CFG);
+	process_arg(option->val, opt, val, precedence, PASS_2_CFG);
 	return true;
     }
 
     return false;
 }
 
-bool read_config_file(const char *fname, bool tilde_expand, bool warn_on_error, priority_t precedence)
+bool read_config_file(const char *fname, bool tilde_expand, bool warn_on_error, priority_t precedence,
+	struct option *longopts)
 {
     bool ok = true;
     int lineno = 0;
@@ -128,7 +151,11 @@ bool read_config_file(const char *fname, bool tilde_expand, bool warn_on_error,
 
     if (config_file_name != NULL)
 	xfree(config_file_name);
-    config_file_name = tildeexpand(fname, tilde_expand);
+
+    if (!tilde_expand)
+	config_file_name = xstrdup(fname);
+    else
+	config_file_name = tildeexpand(fname);
 
     fp = fopen(config_file_name, "r");
 
@@ -145,22 +172,22 @@ bool read_config_file(const char *fname, bool tilde_expand, bool warn_on_error,
     {
 	size_t len;
 	char buff[MAXBUFFLEN];
-	
-	memset(buff, '\0', sizeof(buff));		/* for debugging */
-	
+
 	lineno += 1;
 	if (fgets(buff, sizeof(buff), fp) == NULL)
 	    break;
 	len = strlen(buff);
 	if ( buff[0] == '#' || buff[0] == ';' || buff[0] == '\n' )
 	    continue;
-	while (iscntrl((unsigned char)buff[len-1]) || isspace((unsigned char)buff[len-1]))
+	while (len >= 1
+		&& (iscntrl((unsigned char)buff[len-1])
+		    || isspace((unsigned char)buff[len-1])))
 	    buff[--len] = '\0';
 
 	if (DEBUG_CONFIG(1))
 	    fprintf(dbgout, "Testing:  %s\n", buff);
 
-	if (!process_config_option(buff, warn_on_error, precedence))
+	if (!process_config_option(buff, warn_on_error, precedence, longopts))
 	    ok = false;
     }
 
@@ -175,15 +202,15 @@ bool read_config_file(const char *fname, bool tilde_expand, bool warn_on_error,
 }
 
 /* exported */
-bool process_config_files(bool warn_on_error)
+bool process_config_files(bool warn_on_error, struct option *longopts)
 {
     bool ok = true;
     const char *env = getenv("BOGOTEST");
 
     if (!suppress_config_file) {
-	if (!read_config_file(system_config_file, false, warn_on_error, PR_CFG_SITE))
+	if (!read_config_file(system_config_file, false, warn_on_error, PR_CFG_SITE, longopts))
 	    ok = false;
-	if (!read_config_file(user_config_file, true, warn_on_error, PR_CFG_USER))
+	if (!read_config_file(user_config_file, true, warn_on_error, PR_CFG_USER, longopts))
 	    ok = false;
     }
 
