@@ -1,4 +1,4 @@
-/* $Id: find_home.h,v 1.2 2003/09/29 19:37:13 relson Exp $ */
+/* $Id: find_home.h,v 1.3 2005/03/29 03:04:39 relson Exp $ */
 
 /* find_home.h -- library function to figure out the home dir of current user */
 
@@ -30,6 +30,6 @@ find_home_user(const char *username) /*@globals errno@*/;
 
 /*@only@*/
 char *
-tildeexpand(const char *filename, bool expand) /*@globals errno@*/;
+tildeexpand(const char *filename) /*@globals errno@*/;
 
 #endif
