@@ -1,4 +1,4 @@
-/* $Id: format.c,v 1.48 2006/05/29 14:28:00 relson Exp $ */
+/* $Id: format.c,v 1.51 2006/07/04 12:53:30 relson Exp $ */
 
 /*****************************************************************************
 
@@ -195,11 +195,11 @@ static size_t format_float(char *dest, double src,
     else
 	p = (flags & (F_FP_F|F_FP_G)) ? 6 : 2;
     if (flags & F_FP_F)
-	fmt = flags & F_ZERO ? "%0*.*f" : "%*.*f";
+	fmt = (flags & F_ZERO) ? "%0*.*f" : "%*.*f";
     else
-	fmt = flags & F_ZERO ? "%0*.*e" : "%*.*e";
+	fmt = (flags & F_ZERO) ? "%0*.*e" : "%*.*e";
     if (flags & F_FP_G)
-	fmt = flags & F_ZERO ? "%0*.*g" : "%*.*g";
+	fmt = (flags & F_ZERO) ? "%0*.*g" : "%*.*g";
     snprintf (buf, SIZ, fmt, min, p, s);
     return format_string (dest, buf, 0, 0, 0, destend);
 }
@@ -325,7 +325,7 @@ char *convert_format_to_string(char *buff, size_t size, const char *format)
 		*buff++ = '%';
 		break;
 	    case 'A':		/* A - Message Address */
-		buff += format_string(buff, msg_addr.leng != 0 ? (const char *)msg_addr.text : "UNKNOWN", 0, prec, flags, end);
+		buff += format_string(buff, (*msg_addr->text != '\0') ? (const char *)msg_addr->text : "UNKNOWN", 0, prec, flags, end);
 		break;
 	    case 'c':		/* c - classification, e.g. Yes/No, Spam/Ham/Unsure, or YN, SHU, +-? */
 	    {
@@ -347,10 +347,10 @@ char *convert_format_to_string(char *buff, size_t size, const char *format)
 		break;
 	    }
 	    case 'I':		/* M - Message ID */
-		buff += format_string(buff, msg_id.leng != 0 ? (const char *)msg_id.text : "UNKNOWN", 0, prec, flags, end);
+		buff += format_string(buff, (*msg_id->text != '\0') ? (const char *)msg_id->text : "UNKNOWN", 0, prec, flags, end);
 		break;
 	    case 'Q':		/* Q - Queue ID */
-		buff += format_string(buff, queue_id.leng != 0 ? (const char *)queue_id.text : "UNKNOWN", 0, prec, flags, end);
+		buff += format_string(buff, (*queue_id->text != '\0') ? (const char *)queue_id->text : "UNKNOWN", 0, prec, flags, end);
 		break;
 	    case 'p':		/* p - spamicity as a probability */
 	    {
