@@ -1,4 +1,4 @@
-/* $Id: globals.c,v 1.78 2005/06/17 03:31:06 relson Exp $ */
+/* $Id: globals.c,v 1.81 2006/07/03 00:17:46 relson Exp $ */
 
 /*****************************************************************************
 
@@ -30,6 +30,7 @@ bool	logflag;			/* '-l' */
 bool	mbox_mode;			/* '-M' */
 bool	replace_nonascii_characters;	/* '-n' */
 bool	passthrough;			/* '-p' */
+bool	pairs = false;			/* '-P' */
 bool	quiet;				/* '-q' */
 int	query;				/* '-Q' */
 bool	Rtable;				/* '-R' */
@@ -43,6 +44,11 @@ double	ham_cutoff = HAM_CUTOFF;
 double	spam_cutoff;
 double	thresh_update;
 
+uint	min_token_len       = MIN_TOKEN_LEN;
+uint	max_token_len       = MAX_TOKEN_LEN;
+uint	max_multi_token_len = 0;
+uint	multi_token_count   = MUL_TOKEN_CNT;
+
 const char	*update_dir;
 /*@observer@*/
 const char	*stats_prefix;
