@@ -1,4 +1,4 @@
-/* $Id: globals.h,v 1.77 2005/03/09 01:14:46 relson Exp $ */
+/* $Id: globals.h,v 1.83 2006/06/20 22:10:41 relson Exp $ */
 
 #ifndef GLOBALS_H
 #define GLOBALS_H
@@ -13,6 +13,13 @@
 typedef int  bool;
 #endif
 
+extern	bool	fBogotune;
+extern	bool	fBogoutil;
+extern	bool	fBogofilter;
+
+/* for  transactions */
+extern	e_txn	eTransaction;
+
 /* command line options */
 extern	bulk_t	bulk_mode;		/* '-B' */
 extern	bool	suppress_config_file;	/* '-C' */
@@ -20,9 +27,16 @@ extern	bool	nonspam_exits_zero;	/* '-e' */
 extern	bool	fisher;			/* '-f' */
 extern	FILE	*fpin;			/* '-I' */
 extern	bool	logflag;		/* '-l' */
+
+extern	uint	min_token_len;
+extern	uint	max_token_len;
+extern	uint	max_multi_token_len;
+extern	uint	multi_token_count;
+
 extern	bool	mbox_mode;		/* '-M' */
 extern	char	outfname[PATH_LEN];	/* '-O' */
 extern	bool	passthrough;		/* '-p' */
+extern	bool	pairs;			/* '-P' */
 extern	bool	quiet;			/* '-q' */
 extern	int	query;			/* '-Q' */
 extern	bool	Rtable;			/* '-R' */
@@ -88,16 +102,10 @@ extern	double robx;
 extern	double sp_esf;
 extern	double ns_esf;
 
-/* for  bogotune */
-extern	bool fBogotune;
-
 /* for  BerkeleyDB */
 #define	DB_CACHESIZE	4	/* in MB */
 extern	uint	db_cachesize;
 
-/* for  transactions */
-extern	bool	fTransaction;
-
 /* other */
 
 extern FILE  *fpo;
@@ -111,6 +119,9 @@ extern	bool	unsure_stats;	/* true if print stats for unsures */
 #define	WORDLIST_VERSION	".WORDLIST_VERSION"
 extern	uint	wordlist_version;
 
+#define	WORDLIST_ENCODING	".ENCODING"
+extern	e_enc	encoding;
+
 extern	bool	fDie;		/* true if SIGINT or SIGTERM received */
 
 #endif
