@@ -1,4 +1,4 @@
-/* $Id: qp.c,v 1.16 2004/10/09 03:07:23 relson Exp $ */
+/* $Id: qp.c,v 1.18 2005/09/04 14:49:43 relson Exp $ */
 
 /*****************************************************************************
 
@@ -20,7 +20,7 @@ static byte qp_xlate2045[256];
 static byte qp_xlate2047[256];
 
 static int hex_to_bin(byte c) {
-    switch(c) {
+    switch (c) {
 	case '0': return 0;
 	case '1': return 1;
 	case '2': return 2;
@@ -118,10 +118,9 @@ bool qp_validate(const word_t *word, qp_mode mode)
 	byte b = word->text[i];
 	byte v = qp_xlate[b];
 	if (v == 0)
-	    switch(b) {
-		case '\n':
-		case '\r':
-		case '=':
+	    switch (b) {
+		case '=': /* allowed in encoded words,
+			     but must be encoded */
 		    break;
 		default:
 		    return false;
