@@ -1,4 +1,4 @@
-/* $Id: token.c,v 1.123 2005/09/05 19:06:32 relson Exp $ */
+/* $Id: token.c,v 1.153 2006/11/26 16:34:30 relson Exp $ */
 
 /*****************************************************************************
 
@@ -27,25 +27,20 @@ AUTHOR:
 #include "token.h"
 #include "xmemrchr.h"
 
-#define	MAX_PREFIX_LEN 	  5		/* maximum length of prefix     */
 #define	MSG_COUNT_PADDING 2 * 10	/* space for 2 10-digit numbers */
 
 /* Local Variables */
 
-byte	msg_addr_text[MAXTOKENLEN + D];
-byte	msg_id_text  [MAXTOKENLEN * 3 + D];
-byte	queue_id_text[MAXTOKENLEN + D];
-byte	ipsave_text[MAXTOKENLEN + D];
-
-word_t	msg_addr = { 0, msg_addr_text};	/* First IP Address in Received: statement */
-word_t	msg_id   = { 0, msg_id_text};	/* Message ID */
-word_t	queue_id = { 0, queue_id_text};	/* Message's first queue ID */
+word_t	*msg_addr;	/* First IP Address in Received: statement */
+word_t	*msg_id;	/* Message ID */
+word_t	*queue_id;	/* Message's first queue ID */
 
 static token_t save_class = NONE;
-static word_t ipsave = { 0, ipsave_text};
+static word_t *ipsave;
 
-byte yylval_text[MAXTOKENLEN + MAX_PREFIX_LEN + MSG_COUNT_PADDING + D];
-static word_t yylval = { 0, yylval_text };
+static byte  *yylval_text;
+static size_t yylval_text_size;
+static word_t yylval;
 
 static word_t *w_to   = NULL;	/* To:          */
 static word_t *w_from = NULL;	/* From:        */
@@ -54,6 +49,8 @@ static word_t *w_subj = NULL;	/* Subject:     */
 static word_t *w_recv = NULL;	/* Received:    */
 static word_t *w_head = NULL;	/* Header:      */
 static word_t *w_mime = NULL;	/* Mime:        */
+static word_t *w_ip   = NULL;	/* ip:          */
+static word_t *w_url  = NULL;	/* url:         */
 
 /* Global Variables */
 
@@ -65,12 +62,53 @@ static uint32_t token_prefix_len;
 #define NONBLANK "spc:invalid_end_of_header"
 static word_t *nonblank_line = NULL;
 
+static uint tok_count         = 0;
+static uint init_token        = 1;
+static word_t *p_multi_words  = NULL;
+static byte   *p_multi_buff   = NULL;
+static byte   *p_multi_text   = NULL;
+static word_t **w_token_array = NULL;
+
 /* Function Prototypes */
 
-void token_clear(void);
+static void    token_clear(void);
+static token_t parse_new_token(word_t *token);
+static void    add_token_to_array(word_t *token);
+static void    build_token_from_array(word_t *token);
+static uint    token_copy_leng(const char *str, uint leng, byte *dest);
 
 /* Function Definitions */
 
+static void init_token_array(void)
+{
+    uint i;
+    byte *text;
+    word_t *words;
+		    
+    p_multi_words = calloc( max_token_len, sizeof(word_t) );
+    p_multi_buff  = malloc( max_multi_token_len+D );
+    p_multi_text  = calloc( max_token_len+1+D, multi_token_count );
+    w_token_array = calloc( multi_token_count, sizeof(*w_token_array) );
+
+    text = p_multi_text;
+    words = p_multi_words;
+
+    for (i = 0; i < multi_token_count; i += 1) {
+	words->leng = 0;
+	words->text = text;
+	w_token_array[i] = words;
+	words += 1;
+	text += max_token_len+1+D;
+    }
+}
+
+static void free_token_array(void)
+{
+    free(p_multi_words);
+    free(p_multi_text );
+    free(w_token_array);
+}
+
 static void token_set( word_t *token, byte *text, uint leng )
 {
     token->leng = leng;
@@ -83,27 +121,69 @@ static inline void token_copy( word_t *dst, word_t *src )
     token_set(dst, src->text, src->leng);
 }
 
-static void build_prefixed_token( word_t *token, uint32_t token_size,
-				  word_t *prefix, 
-				  byte *text, uint32_t leng )
+static void build_prefixed_token( word_t *token, word_t *prefix, 
+				  word_t *temp, uint32_t temp_size )
 {
-    uint p = (prefix == NULL) ? 0 : prefix->leng;
-    uint l = leng + p;
+    uint pfx_len = (prefix == NULL) ? 0 : prefix->leng;
+    uint len = token->leng + pfx_len;
+    
+    if (len >= temp_size)
+	len = temp_size - pfx_len - 1;
 
-    if (l >= token_size)
-	l = token_size - p - 1;
-
-    token->leng = l;
+    temp->leng = len;
 
     /* copy prefix, if there is one */
     if (prefix != NULL)
-	memcpy(token->text, prefix->text, p);
+	memcpy(temp->text, prefix->text, pfx_len);
 
-    memcpy(token->text + p, text, l-p);		/* include nul terminator */
-    token->text[token->leng] = '\0';		/* ensure nul termination */
+    memcpy(temp->text + pfx_len, token->text, len-pfx_len);
+    Z(temp->text[temp->leng]);
+
+    token->leng = temp->leng;
+    token->text = temp->text;
 }
 
-token_t get_token(word_t **token)
+#define WRAP(n)	((n) % multi_token_count)
+
+token_t get_token(word_t *token)
+{
+    token_t cls;
+    
+    bool fSingle = (tok_count < 2 ||
+		    tok_count <= init_token ||
+		    multi_token_count <= init_token);
+
+    if (fSingle) {
+	cls = parse_new_token(token);
+
+	if (multi_token_count > 1)
+	    add_token_to_array(token);
+    }
+    else {
+	cls = TOKEN;
+	build_token_from_array(token);
+    }
+
+    if (token_prefix != NULL) {
+	/* IP addresses get special prefix */
+	if (save_class != IPADDR) {
+	    build_prefixed_token(token, token_prefix, &yylval, yylval_text_size);
+	}
+	else {
+
+	    word_t *prefix = (wordlist_version >= IP_PREFIX) ? w_ip : w_url;
+	    build_prefixed_token(token, prefix, ipsave, max_token_len);
+	}
+
+	/* if excessive length caused by prefix, get another token */
+	if (fSingle && token->leng > max_token_len)
+	    cls = get_token(token);
+    }
+
+    return cls;
+}
+
+token_t parse_new_token(word_t *token)
 {
     token_t cls = NONE;
     unsigned char *cp;
@@ -112,13 +192,13 @@ token_t get_token(word_t **token)
     /* If saved IPADDR, truncate last octet */
     if ( block_on_subnets && save_class == IPADDR )
     {
-	byte *t = xmemrchr(ipsave.text, '.', ipsave.leng);
+	byte *t = xmemrchr(ipsave->text, '.', ipsave->leng);
 	if (t == NULL)
 	    save_class = NONE;
 	else
 	{
-	    ipsave.leng = (uint) (t - ipsave.text);
-	    token_set( &yylval, ipsave.text, ipsave.leng);
+	    ipsave->leng = (uint) (t - ipsave->text);
+	    token_set( &yylval, ipsave->text, ipsave->leng);
 	    cls = save_class;
 	    done = true;
 	}
@@ -127,13 +207,18 @@ token_t get_token(word_t **token)
     while (!done) {
 	uint leng;
 	byte *text;
+
 	cls = (*lexer->yylex)();
-	
-	leng = (uint)   *lexer->yyleng;
-	text = (byte *) *lexer->yytext;
-	
+
+	token->leng = (uint)   *lexer->yyleng;
+	token->text = (byte *) *lexer->yytext;
+	Z(token->text[token->leng]);	/* for easier debugging - removable */
+
+	leng = token->leng;
+	text = token->text;
+
 	if (DEBUG_TEXT(2)) {
-	    word_puts(&yylval, 0, dbgout);
+	    word_puts(token, 0, dbgout);
 	    fputc('\n', dbgout);
 	}
  
@@ -143,7 +228,7 @@ token_t get_token(word_t **token)
 	switch (cls) {
 
 	case EOH:	/* end of header - bogus if not empty */
-	    if (leng > MAXTOKENLEN)
+	    if (leng > max_token_len)
 		continue;
 
 	    if (msg_state->mime_type == MIME_MESSAGE)
@@ -173,27 +258,27 @@ token_t get_token(word_t **token)
 		    lst = in;
 		}
 	    }
+
 	    if (fst != NULL && lst != NULL && lst - fst  > 3) {
 		byte *ot = fst;
 		*ot++ = '-';
 		*ot++ = '#';
 		for (in = lst; *in != '\0'; in += 1, ot += 1)
 		    *ot = *in;
-		leng = (uint) (ot - st);
+		token->leng = leng = (uint) (ot - st);
 	    }
-	    text[leng] = '\0';		/* ensure nul termination */
-	    build_prefixed_token(&yylval, sizeof(yylval_text), token_prefix, text, leng);
+	    Z(token->text[token->leng]);	/* for easier debugging - removable */
 	}
 	break;
 
 	case HEADKEY:
 	{
-	    if (!header_line_markup)
+	    if (!header_line_markup || *text == '\0')
 		continue;
 	    else {
 		const char *delim = strchr((const char *)text, ':');
 		leng = (uint) (delim - (const char *)text);
-		if (leng > MAXTOKENLEN)
+		if (leng > max_token_len)
 		    continue;
 		token_set( &yylval, text, leng);
 	    }
@@ -202,9 +287,15 @@ token_t get_token(word_t **token)
 	/*@fallthrough@*/
 
 	case TOKEN:	/* ignore anything when not reading text MIME types */
-	    if (leng > MAXTOKENLEN)
+	    if (leng < min_token_len)
 		continue;
-	    build_prefixed_token(&yylval, sizeof(yylval_text), token_prefix, text, leng);
+	case MONEY:	/* 2 character money is OK */
+	    if (leng > max_token_len)
+		continue;
+
+	    token->text = text;
+	    token->leng = leng;
+
 	    if (token_prefix == NULL) {
 		switch (msg_state->mime_type) {
 		case MIME_TEXT:
@@ -215,6 +306,7 @@ token_t get_token(word_t **token)
 		case MIME_MESSAGE:
 		case MIME_APPLICATION:
 		case MIME_IMAGE:
+		    continue;
 		default:
 		    continue;
 		}
@@ -224,7 +316,7 @@ token_t get_token(word_t **token)
 	case MESSAGE_ID:
 	    /* special token;  saved for formatted output, but not returned to bogofilter */
 	    /** \bug: the parser MUST be aligned with lexer_v3.l! */
-	    if (leng < sizeof(msg_id_text))
+	    if (leng < max_token_len)
 	    {
 		while (!isspace(text[0])) {
 		    text += 1;
@@ -234,16 +326,15 @@ token_t get_token(word_t **token)
 		    text += 1;
 		    leng -= 1;
 		}
-		token_set( &yylval, text, leng);
-		token_copy( &msg_id, &yylval );
+		token_set( msg_id, text, leng);
 	    }
-	continue;
+	    continue;
 
 	case QUEUE_ID:
 	    /* special token;  saved for formatted output, but not returned to bogofilter */
 	    /** \bug: the parser MUST be aligned with lexer_v3.l! */
-	    if (queue_id.leng == 0 &&
-		leng < sizeof(queue_id_text) )
+	    if (*queue_id->text == '\0' &&
+		leng < max_token_len )
 	    {
 		while (isspace(text[0])) {
 		    text += 1;
@@ -257,8 +348,16 @@ token_t get_token(word_t **token)
 		    text += 1;
 		    leng -= 1;
 		}
-		token_set( &yylval, text, leng);
-		token_copy( &queue_id, &yylval );
+		if (text[0] == '<') {
+		    text += 1;
+		    leng -= 1;
+		}
+		if (text[leng-1] == '>') {
+		    leng -= 1;
+		}
+		leng = min(queue_id->leng, leng);
+		memcpy( queue_id->text, text, leng );
+		Z(queue_id->text[leng]);
 	    }
 	    continue;
 
@@ -267,14 +366,17 @@ token_t get_token(word_t **token)
 	    /* trim brackets */
 	    text += 1;
 	    leng -= 2;
+	    Z(text[leng]);	/* for easier debugging - removable */
 	    token_set( &yylval, text, leng);
 	    /* if top level, no address, not localhost, .... */
 	    if (token_prefix == w_recv &&
 		msg_state->parent == NULL && 
-		msg_addr.leng == 0 &&
-		strcmp((char *)text, "127.0.0.1") != 0) {
+		*msg_addr->text == '\0' &&
+		strcmp((char *)text, "127.0.0.1") != 0)
+	    {
 		/* Not guaranteed to be the originating address of the message. */
-		token_copy( &msg_addr, &yylval );
+		memcpy( msg_addr->text, yylval.text, min(msg_addr->leng, yylval.leng)+D );
+		Z(msg_addr->text[yylval.leng]);
 	    }
 	}
 
@@ -283,8 +385,6 @@ token_t get_token(word_t **token)
 	case IPADDR:
 	    if (block_on_subnets)
 	    {
-		const char *ptext = (wordlist_version >= IP_PREFIX) ? "ip:" : "url:";
-		word_t *prefix = word_news(ptext);
 		int q1, q2, q3, q4;
 		/*
 		 * Trick collected by ESR in real time during John
@@ -304,15 +404,19 @@ token_t get_token(word_t **token)
 			    q1 & 0xff, q2 & 0xff, q3 & 0xff, q4 & 0xff);
 		leng = strlen((const char *)text);
 
-		build_prefixed_token(&ipsave, sizeof(ipsave_text), prefix, text, leng);
-		token_copy( &yylval, &ipsave );
-		word_free(prefix);
+		token->text = text;
+		token->leng = leng;
+
+		token_copy( ipsave, token );
 
 		save_class = IPADDR;
-		*token = &yylval;
+
 		return (cls);
 	    }
-	    build_prefixed_token(&yylval, sizeof(yylval_text), token_prefix, text, leng);
+
+	    token->text = text;
+	    token->leng = leng;
+
 	    break;
 
 	case NONE:		/* nothing to do */
@@ -320,6 +424,7 @@ token_t get_token(word_t **token)
 
 	case MSG_COUNT_LINE:
 	    msg_count_file = true;
+	    multi_token_count = 1;
 	    header_line_markup = false;
 	    token_prefix = NULL;
 	    lexer = &msg_count_lexer;
@@ -342,43 +447,142 @@ token_t get_token(word_t **token)
 	}
 
 	/* eat all long words */
-	if (yylval.leng <= MAXTOKENLEN)
+	if (token->leng <= max_token_len)
 	    done = true;
     }
 
    if (!msg_count_file) {
 	/* Remove trailing blanks */
 	/* From "From ", for example */
-	while (yylval.leng > 1 && yylval.text[yylval.leng-1] == ' ') {
-	    yylval.leng -= 1;
-	    yylval.text[yylval.leng] = (byte) '\0';
+	while (token->leng > 1 && token->text[token->leng-1] == ' ') {
+	    token->leng -= 1;
+	    token->text[token->leng] = (byte) '\0';
 	}
 
 	/* Remove trailing colon */
-	if (yylval.leng > 1 && yylval.text[yylval.leng-1] == ':') {
-	    yylval.leng -= 1;
-	    yylval.text[yylval.leng] = (byte) '\0';
+	if (token->leng > 1 && token->text[token->leng-1] == ':') {
+	    token->leng -= 1;
+	    token->text[token->leng] = (byte) '\0';
 	}
 
 	if (replace_nonascii_characters) {
 	    /* replace nonascii characters by '?'s */
-	    for (cp = yylval.text; cp < yylval.text+yylval.leng; cp += 1)
+	    for (cp = token->text; cp < token->text+token->leng; cp += 1)
 		*cp = casefold_table[*cp];
 	}
     }
 
-    *token = &yylval;
-
     return(cls);
 }
 
+/* save token in token array */
+
+static void add_token_to_array(word_t *token)
+{
+    word_t *w = w_token_array[WRAP(tok_count)];
+
+    w->leng = token->leng;
+    memcpy(w->text, token->text, w->leng);
+    Z(w->text[w->leng]);	/* for easier debugging - removable */
+
+    if (DEBUG_MULTI(1))
+	fprintf(stderr, "%s:%d  %2s  %2d %2d %p %s\n", __FILE__, __LINE__,
+		"", tok_count, w->leng, w->text, w->text);
+
+    tok_count += 1;
+    init_token = 1;
+
+    return;
+}
+
+static void build_token_from_array(word_t *token)
+{
+    int tok;
+
+    const char *sep = "";
+    uint  leng;
+    byte *dest;
+
+    leng = init_token;
+    for ( tok = init_token; tok >= 0; tok -= 1 ) {
+	uint idx = tok_count - 1 - tok;
+	leng += strlen((char *) w_token_array[WRAP(idx)]->text);
+    }
+
+    if (leng > max_multi_token_len)
+	leng = max_multi_token_len;
+
+    token->leng = leng;
+    token->text = dest = p_multi_buff;
+
+    for ( tok = init_token; tok >= 0; tok -= 1 ) {
+	uint  idx = tok_count - 1 - tok;
+	uint  len = w_token_array[WRAP(idx)]->leng;
+	byte *str = w_token_array[WRAP(idx)]->text;
+
+	if (DEBUG_MULTI(1))
+	    fprintf(stderr, "%s:%d  %2d  %2d %2d %p %s\n", __FILE__, __LINE__,
+		    idx, tok_count, len, str, str);
+	
+	len = token_copy_leng((const char *)sep, leng, dest);
+	leng -= len;
+	dest += len;
+
+	len = token_copy_leng((const char *)str, leng, dest);
+	leng -= len;
+	dest += len;
+
+	sep = "*";
+    }
+
+    Z(token->text[token->leng]);	/* for easier debugging - removable */
+    init_token += 1;			/* progress to next multi-token */
+
+    return;
+}
+
+static uint token_copy_leng(const char *str, uint leng, byte *dest)
+{
+    uint len = strlen(str);
+    if (leng < len)
+	len  = leng;
+    if (len != 0)
+	memcpy(dest, str, len);
+    return (uint) len;
+}
+
 void token_init(void)
 {
+    static bool fTokenInit = false;
+
     yyinit();
 
-    token_clear();
+    if ( fTokenInit) {
+	token_clear();
+    }
+    else {
+	fTokenInit = true;
+
+	if (max_multi_token_len == 0)
+	    max_multi_token_len = (max_token_len+1) * multi_token_count + MAX_PREFIX_LEN;
+
+	yylval_text_size = max_multi_token_len + MSG_COUNT_PADDING;
+
+	yylval_text = (byte *) malloc( yylval_text_size+D );
+	yylval.leng   = 0;
+	yylval.text   = yylval_text;
+
+	/* First IP Address in Received: statement */
+	msg_addr = word_new( NULL, max_token_len );
+
+	/* Message ID */
+	msg_id = word_new( NULL, max_token_len * 3 );
+
+	/* Message's first queue ID */
+	queue_id = word_new( NULL, max_token_len );
+
+	ipsave = word_new( NULL, max_token_len );
 
-    if (w_to == NULL) {
 	/* word_new() used to avoid compiler complaints */
 	w_to   = word_news("to:");	/* To:          */
 	w_from = word_news("from:");	/* From:        */
@@ -387,7 +591,12 @@ void token_init(void)
 	w_recv = word_news("rcvd:");	/* Received:    */
 	w_head = word_news("head:");	/* Header:      */
 	w_mime = word_news("mime:");	/* Mime:        */
+	w_ip   = word_news("ip:");	/* ip:          */
+	w_url  = word_news("url:");	/* url:         */
 	nonblank_line = word_news(NONBLANK);
+
+	/* do multi-word token initializations */
+	init_token_array();
     }
 
     return;
@@ -396,10 +605,13 @@ void token_init(void)
 void clr_tag(void)
 {
     token_prefix = NULL;
+    tok_count = 0;
 }
 
 void set_tag(const char *text)
 {
+    word_t *old_prefix = token_prefix;
+
     if (!header_line_markup)
 	return;
 
@@ -448,14 +660,18 @@ void set_tag(const char *text)
 	    word_puts(token_prefix, 0, dbgout);
 	fputc('\n', dbgout);
     }
+
+    /* discard tokens when prefix changes */
+    if (old_prefix != NULL && old_prefix != token_prefix)
+	tok_count = 0;
+
     return;
 }
 
 void set_msg_id(byte *text, uint leng)
 {
-    if (leng >= sizeof(msg_id_text))	/* Limit length */
-	leng  = sizeof(msg_id_text) - 1;
-    token_set( &msg_id, text, leng );
+    (void) leng;		/* suppress compiler warning */
+    token_set( msg_id, text, msg_id->leng );
 }
 
 #define WFREE(n)	word_free(n); n = NULL
@@ -470,14 +686,22 @@ void token_cleanup()
     WFREE(w_recv);
     WFREE(w_head);
     WFREE(w_mime);
+    WFREE(w_ip);
+    WFREE(w_url);
     WFREE(nonblank_line);
 
     token_clear();
+
+    /* do multi-word token cleanup */
+    free_token_array();
 }
 
 void token_clear()
 {
-    msg_addr.leng = 0;
-    msg_id.leng   = 0;
-    queue_id.leng = 0;
+    if (msg_addr != NULL)
+    {
+	*msg_addr->text = '\0';
+	*msg_id->text   = '\0';
+	*queue_id->text = '\0';
+    }
 }
