@@ -1,4 +1,4 @@
-/* $Id: wordlists.h,v 1.22 2005/04/04 11:16:22 relson Exp $ */
+/* $Id: wordlists.h,v 1.24 2005/05/19 11:45:28 m-a Exp $ */
 
 /*  constants and declarations for wordlists */
 
@@ -15,13 +15,19 @@ extern size_t	   cSeparate;
 
 typedef	char FILEPATH[PATH_LEN];
 
-/*@null@*/ 
+/*@null@*/
 extern wordlist_t *word_lists;
 
 void incr_wordlist_mode(void);
 void set_wordlist_mode(const char *filepath);
 bool configure_wordlist(const char *val);
 
+/**
+ * initialize wordlist the same way as open_wordlists does, like
+ * beginning a transaction and reading message counts
+ */
+void begin_wordlist(wordlist_t *list);
+
 void open_wordlists(dbmode_t mode);
 bool close_wordlists(bool commit);
 bool query_wordlists_closed(void);
